#include <math.h>
#include <octave/oct.h>
#include <octave/parse.h>
#include <octave/f77-fcn.h>

/* 

An octave gateway routine to the FORTRAN code hwscrt.f.
The code can be found at

http://cm.bell-labs.com/netlib/fishpack/

Written by:
-- 
John L. Weatherwax                2005-10-29

email: wax@alum.mit.edu

Please send comments and especially bug reports to the
above email address.

-----
*/ 

extern "C"
{
  F77_RET_T F77_FUNC(hwscrt,HWSCRT)(const double&, const double&, const int&, const int&, double*, double*,
                                    const double&, const double&, const int&, const int&, double*, double*,
                                    const double&, double*, const int&, double&, int&, double*);
}

static octave_function *hwscrtg_oct_fcn;

DEFUN_DLD (hwscrtg, args, nargout,
"u = hwscrtg(a,b,nx,xbc,abd,bbd,c,d,ny,ybc,cbd,dbd,lambda,inhom_f):\n\
Solve the two-dimensional inhomogenous Helmholtz equation:\n\
\n\
u_{xx} + u_{yy} + lambda u = f(x,y)\n\
\n\
in cartesian coordinates using a five-point finite difference approximation.\n\
Individual inputs are as follows\n\
\n\
A = left end point of x domain,\n\
B = right end point of x domain,\n\
M = number of cells in the x direction,\n\
MBDCND = the type of boundary condition in the x dimension (choices are),\n\
       = 0  if the solution is periodic in x, i.e., u(i,j) = u(m+i,j).\n\
       = 1  if the solution is specified at x = A and x = B.\n\
       = 2  if the solution is specified at x = A and the derivative of\n\
            the solution with respect to x is specified at x = B.\n\
       = 3  if the derivative of the solution with respect to x is\n\
            specified at x = A and x = B.\n\
       = 4  if the derivative of the solution with respect to x is\n\
            specified at x = A and the solution is specified at x = B.\n\
BDA = A one-dimensional array of length N+1 that specifies the values\n\
of the derivative of the solution with respect to x at x = A.\n\
When MBDCND = 3 or 4,\n\
\n\
BDA(j) = (d/dx)U(A,Y(j)), j = 1,2,...,N+1.\n\
BDB = boundary data for the right face of the domain (x=B),\n\
C = lower end point of y domain,\n\
D = upper end point of y domain,\n\
N = number of cells in the y direction,\n\
NBDCND = the type of boundary condition in the y dimension (see below for choices),\n\
BDC = boundary data for the bottom face of the domain (y=C),\n\
BDD = boundary data for the top face of the domain (y=D),")
{
  octave_value_list retval;

  if( args.length() != 14 ){ print_usage("hwscrtg"); return retval; }
  if( nargout > 2 ) { print_usage("hwscrtg"); return retval; }

  // Extract the user input:
  //
  double A = args(0).double_value();
  double B = args(1).double_value();
  int M = args(2).int_value();
  int MBDCND = args(3).int_value(); 
  //octave_stdout << "A = " << A << "\n"; 
  //octave_stdout << "B = " << B << "\n"; 
  //octave_stdout << "M = " << M << "\n"; 
  //octave_stdout << "MBDCND = "  << MBDCND << "\n";
  ColumnVector bda_v( args(4).vector_value() ); 
  //octave_stdout << "bda_v = " << bda_v << "\n"; 
  double* BDA = bda_v.fortran_vec();
  ColumnVector bdb_v( args(5).vector_value() ); 
  //octave_stdout << "bdb_v = " << bdb_v << "\n"; 
  double* BDB = bdb_v.fortran_vec();
  double C = args(6).double_value();
  double D = args(7).double_value();
  int N = args(8).int_value();
  int NBDCND = args(9).int_value();
  //octave_stdout << "C = " << C << "\n"; 
  //octave_stdout << "D = " << D << "\n"; 
  //octave_stdout << "N = " << N << "\n"; 
  //octave_stdout << "NBDCND = "  << NBDCND << "\n";  
  ColumnVector bdc_v( args(10).vector_value() ); 
  double* BDC = bdc_v.fortran_vec();
  ColumnVector bdd_v( args(11).vector_value() ); 
  double* BDD = bdd_v.fortran_vec();
  double ELMBDA = args(12).double_value(); 
  Matrix F_m( args(13).matrix_value() ); 
  double* F = F_m.fortran_vec(); 

  int IDIMF = M+1; 
  //octave_stdout << "IDIMF = " << IDIMF << "\n"; 
  int nw = (int) ( 4*(N+1) + (13+log2(N+1))*(M+1) + 1 );
  //octave_stdout << "nw = " << nw << "\n"; 
  OCTAVE_LOCAL_BUFFER( double, WP, nw ); 

  double PERTRB; 
  int IERROR; 
  F77_FUNC(hwscrt,HWSCRT)(A,B,M,MBDCND,BDA,BDB,C,D,N,NBDCND,BDC,BDD,ELMBDA,F,IDIMF,PERTRB,IERROR,WP);
  //octave_stdout << "IERROR = " << IERROR << "\n"; 
  switch( IERROR ){ 
  case 0:
      /* do nothing ... no error */ break;
  case 1:
      error( "A > B" ); return retval; break; 
  case 2:
      error( "MBDCND must be between 0 and 4 inclusive" ); return retval; break; 
  case 3:
      error( "C > D" ); return retval; break; 
  case 4:
      error( "N must be greater than 3" ); return retval; break; 
  case 5:
      error( "NBDCND must be between 0 and 4 inclusive" ); return retval; break; 
  case 6:
      error( "lambda must be negative" ); return retval; break; 
  case 7:
      error( "idimf must be greater than M+1" ); return retval; break; 
  case 8:
      error( "M must be greater than 3" ); return retval; break; 
  default: 
    error( "Unknown value of IERROR" ); return retval; 
  }

  /* Fill the solution matrix from the output F */ 
  retval.append(F_m); 
  //octave_stdout << "PERTRB = " << PERTRB << "\n"; 
  //retval.append(PERTRB);
  return retval;   
}
